/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2009 runlevel3 GmbH
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of CyberClockControl.
 *
 * CyberClockControl is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CyberClockControl is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CyberClockControl; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Qt Creator / Qt 4.5.0
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2009 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v2
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <qglobal.h>
#include "i2c_usb.h"
#ifdef Q_OS_UNIX
    #include "usb.h"
#endif
#ifdef Q_OS_WIN
    #include "libusb-win32-0.1.12.1\include\usb.h"
#endif

#define DEVICESTR "BUS_%03d-DEV_%03d-VID_%04X-PID_%04X"


Ui::MainWindow *ui;
struct usb_bus *bus;
struct usb_device *dev;
usb_dev_handle *handle = NULL;
bool chip_found;
extern char config_bob[256];


void do_usb_ui_init(Ui::MainWindow *_ui)
{
    ui = _ui;
    usb_init();
}

void do_usb_error_display()
{
    char tmps[256];

    strncpy(tmps, "Error: ", sizeof(tmps));
    ui->statusBar->showMessage(strncat(tmps, usb_strerror(), sizeof(tmps) - strlen(tmps)));
}

int do_usb_scan()
{
    usb_dev_handle *udev;
    char id[256];
    char tmps[256];
    int rv;

    chip_found = false;
    ui->USBcomboBox->clear();
    usb_find_busses();
    usb_find_devices();

    if (!usb_busses) {
        ui->USBcomboBox->addItem("No USB subsystem found on this machine!");
        do_usb_error_display();
        return -1;
    }
    for (bus = usb_busses; bus; bus = bus->next) {
        for (dev = bus->devices; dev; dev = dev->next) {
            snprintf(id, sizeof(id), DEVICESTR, atoi(bus->dirname), dev->devnum, dev->descriptor.idVendor, dev->descriptor.idProduct);
            udev = usb_open(dev);
            if (udev) {
                if (dev->descriptor.iManufacturer) {
                    rv = usb_get_string_simple(udev, dev->descriptor.iManufacturer, tmps, sizeof(tmps));
                    if (rv > 0) snprintf(id + strlen(id), sizeof(id) - strlen(id), " - %s", tmps);
                }
                if (dev->descriptor.iProduct) {
                    rv = usb_get_string_simple(udev, dev->descriptor.iProduct, tmps, sizeof(tmps));
                    if (rv > 0) snprintf(id + strlen(id), sizeof(id) - strlen(id), " - %s", tmps);
                }
                usb_close(udev);
            }
            ui->USBcomboBox->addItem(id);
            if ((dev->descriptor.idVendor == I2C_TINY_USB_VID) && (dev->descriptor.idProduct == I2C_TINY_USB_PID)) {
                if (!handle) {
                    ui->USBcomboBox->setCurrentIndex(ui->USBcomboBox->count() - 1);
                    if (do_usb_open() >= 0) chip_found = true;
                }
            }
        }
    }
    return 0;
}

int do_usb_open()
{
    char tmps[256];
    int rv, bus_nr, dev_nr, vid, pid;

    if (handle) do_usb_close();

    sscanf(ui->USBcomboBox->currentText().toAscii(), DEVICESTR, &bus_nr, &dev_nr, &vid, &pid);
    if ((vid != I2C_TINY_USB_VID) || (pid != I2C_TINY_USB_PID)) {
        ui->statusBar->showMessage("Error: wrong VID/PID, only i2c-tiny-usb interface supported");
        return -1;
    }
    for (bus = usb_busses; bus; bus = bus->next) {
        for (dev = bus->devices; dev; dev = dev->next) {
            snprintf(tmps, sizeof(tmps), DEVICESTR, atoi(bus->dirname), dev->devnum, dev->descriptor.idVendor, dev->descriptor.idProduct);
            if (!strncmp(tmps, ui->USBcomboBox->currentText().toAscii(), 33)) {
                if ((dev->descriptor.idVendor != I2C_TINY_USB_VID) || (dev->descriptor.idProduct != I2C_TINY_USB_PID)) {
                    ui->statusBar->showMessage("Error: wrong VID/PID, only i2c-tiny-usb interface supported");
                    return -1;
                }
                if(!(handle = usb_open(dev))) {
                    strncpy(tmps, "Error: ", sizeof(tmps));
                    ui->statusBar->showMessage(strncat(tmps, usb_strerror(), sizeof(tmps) - strlen(tmps)));
                    return -1;
                }
#ifdef Q_OS_UNIX
                // Get exclusive access to interface 0. Does not work under windows.
                if (usb_claim_interface(handle, 0) != 0) {
                    strncpy(tmps, "Warning: ", sizeof(tmps));
                    ui->statusBar->showMessage(strncat(tmps, usb_strerror(), sizeof(tmps) - strlen(tmps)));
                }
#endif
                rv = i2c_read_multi(CY22150_ADDR, 0, sizeof(config_bob), config_bob);
                switch (rv) {
                    case 0:
                        ui->statusBar->showMessage("Info: Device configuration successfully read.");
                        return 0;
                        break;
                    case -1:
                        return -1;
                        break;
                    case -2:
                        ui->statusBar->showMessage("Error: Device not found at this interface!");
                        return -1;
                        break;
                    case -3:
                        ui->statusBar->showMessage("Error: Device not complete readable!");
                        return -1;
                        break;
                    default:
                        strncpy(tmps, "Error: ", sizeof(tmps));
                        ui->statusBar->showMessage(strncat(tmps, usb_strerror(), sizeof(tmps) - strlen(tmps)));
                        return -1;
                        break;
                }
            }
        }
    }
    return -1;
}

int do_usb_close()
{
    if (!handle) return 0;
#ifdef Q_OS_UNIX
    if (usb_release_interface(handle, 0) != 0) {
        ui->statusBar->showMessage(usb_strerror());
    }
#endif
    usb_close(handle);
    handle = NULL;
    return 0;
}

// get transaction status
int i2c_tiny_usb_get_status(void)
{
    int rv;
    char status;

    if (!handle) return 0;

    rv = usb_control_msg(handle, USB_CTRL_IN, CMD_GET_STATUS, 0, 0, &status, 1, 1000);
    if (rv < 0) return rv;
    return status;
}

// read 1..256 bytes from the i2c device
int i2c_read_multi(unsigned char chip_addr, char start_addr, int length, char *result)
{
    int cnt;

    if (!handle) return 0;

    if (length <= 0) return -1;
    if (length > 256) length = 256;

    if (usb_control_msg(handle, USB_CTRL_OUT, CMD_I2C_IO + CMD_I2C_BEGIN, 0, chip_addr, &start_addr, 1, 1000) < 1) return -4;

    if (i2c_tiny_usb_get_status() != STATUS_ADDRESS_ACK) return -2;

    for (cnt = 0; cnt < length; cnt++) {
        ui->LOADSAVEprogressBar->setValue(cnt * ui->LOADSAVEprogressBar->maximum() / length);
        ui->LOADSAVEprogressBar->repaint();
        if (cnt == length - 1) {
            if (usb_control_msg(handle, USB_CTRL_IN, CMD_I2C_IO + CMD_I2C_END, I2C_M_RD, chip_addr, &result[cnt], 1, 1000) < 1) return -5;
        } else {
            if (usb_control_msg(handle, USB_CTRL_IN, CMD_I2C_IO, I2C_M_RD, chip_addr, &result[cnt], 1, 1000) < 1) return -6;
        }
        if (i2c_tiny_usb_get_status() != STATUS_ADDRESS_ACK) return -3;
    }

    if (i2c_tiny_usb_get_status() != STATUS_ADDRESS_ACK) return -3;

    ui->LOADSAVEprogressBar->setValue(ui->LOADSAVEprogressBar->maximum());
    return 0;
}

// write one byte register address to chip
int i2c_write_addr(unsigned char chip_addr, char data)
{
    if (!handle) return 0;

    if(usb_control_msg(handle, USB_CTRL_OUT, CMD_I2C_IO + CMD_I2C_BEGIN + CMD_I2C_END, 0, chip_addr, &data, 1, 1000) < 1) return -1;

    if(i2c_tiny_usb_get_status() != STATUS_ADDRESS_ACK) return -2;

    return 0;
}

// write one byte to chip
int i2c_write_byte(unsigned char chip_addr, char start_addr, char data)
{
    char buffer[2];

    if (!handle) return 0;

    buffer[0] = start_addr;
    buffer[1] = data;

    if (usb_control_msg(handle, USB_CTRL_OUT, CMD_I2C_IO + CMD_I2C_BEGIN + CMD_I2C_END, 0, chip_addr, buffer, 2, 1000) < 1) return -4;

    if (i2c_tiny_usb_get_status() != STATUS_ADDRESS_ACK) return -2;

    return 0;
}
