/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2009 runlevel3 GmbH
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of CyberClockControl.
 *
 * CyberClockControl is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CyberClockControl is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CyberClockControl; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Qt Creator / Qt 4.5.0
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2009 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v2
 * @link      http://www.runlevel3.de runlevel3
 **/


#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>

typedef struct pll_settings
{
    double f_ref;                       // reference input frequency
    double f1;                          // requested output divider 1
    double f2;                          // requested output divider 2
    double frac;                        // Abweichung vom Sollwert
    int p;                              // pll-internal value p
    int q;                              // pll-internal value q
    int d1;                             // divider DIV1N
    int d2;                             // divider DIV2N
    int f1_importance;                  // wichtigkeit, 1=wichtig
    int f2_importance;
    int iterations;                     // iterations to find settings
} pll_settings;



namespace Ui
{
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

private:
    Ui::MainWindow *ui;
    void fileread();
    void filewrite();
    void fillCheckboxes();
    void fillComboboxes();
    void computePLLfrequency();
    void computeFrequencies();
    void findPLLsettings(pll_settings *pll_vars);
    void setChargePump(pll_settings *pll_vars);
    void setPQD1D2(pll_settings *pll_vars);
    void writeALLconfig();

private slots:
    void on_cbF2_pref_clicked();
    void on_cbF1_pref_clicked();
    void on_leF2_returnPressed();
    void on_leF1_returnPressed();
    void on_cbPLL_extend_toggled(bool checked);
    void on_cbF2_enable_toggled(bool checked);
    void on_leF2_editingFinished();
    void on_leF1_editingFinished();
    void on_sbFreference_valueChanged(int );
    void on_rbDIV2CLK_PLL_clicked();
    void on_rbDIV2CLK_ref_clicked();
    void on_rbDIV1CLK_PLL_clicked();
    void on_rbDIV1CLK_ref_clicked();
    void on_cbPO_clicked();
    void on_anycheckbox_toggled();
    void on_anyclockbutton_toggled();
    void on_anymatrixcombobox_changed();
    void on_anysbvalue_changed();
    void on_actionOpen_triggered();
    void on_actionScan_triggered();
    void on_actionSave_As_triggered();
    void on_actionSave_triggered();
    void on_actionLoad_triggered();
};

#endif // MAINWINDOW_H
